/* Copyright (c) 1998, MpegTV LLC */
#ifndef _voh_api_h
#define _voh_api_h
#ifndef lint
static const char rcsid_voh_api_h[] = "$Header: /net/hopkins/home/tristan/mpeg/dec/RCS/voh_api.h,v 1.93 1999/04/01 10:37:26 tristan Exp $";
#endif

/*
 * Disclaimer of Warranty
 *
 * These software programs are available to the user without any license fee or
 * royalty on an "as is" basis.  Copyright-holder disclaims
 * any and all warranties, whether express, implied, or statuary, including any
 * implied warranties or merchantability or of fitness for a particular
 * purpose.  In no event shall the copyright-holder be liable for any
 * incidental, punitive, or consequential damages of any kind whatsoever
 * arising from the use of these programs.
 *
 * This disclaimer of warranty extends to the user of these programs and user's
 * customers, employees, agents, transferees, successors, and assigns.
 *
 */

/* VOH = Video Output Handler */

#include "mtvp/misc.h"

#include "mtvp/com_api.h"
#include "mtvp/dith_im.h"
#include "mtvp/dither_api.h"
#include "mtvp/vci_api.h"

/*--------------------------------------------------------------------------+
| Configuration																|
+--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------+
| Macros																	|
+--------------------------------------------------------------------------*/
#define VOH_VERSION	0

#define VOH_HANDLER_X	255
#define VOH_HANDLER_SDL	254

/* max depth to match in TrueColor visuals */
#define VOH_FULL_COLOR_MAX_DEPTH_DEFAULT	32

#define VOH_RED		0
#define VOH_GREEN	1
#define VOH_BLUE	2
#define VOH_RGB		3	/* size of arrays indexed by RED, GREEN, BLUE */

#define VOH_CONTROL_DITHER_TYPE					0x0001
#define VOH_CONTROL_SCALE						0x0002
#define VOH_CONTROL_VIDEO_LAYOUT             	0x0004
#define VOH_CONTROL_PEL_ASPECT_RATIO			0x0008
#define VOH_CONTROL_SIZE						0x0010
#define VOH_CONTROL_WINDOW_POSITION				0x0020
#define VOH_CONTROL_EXISTING_WINDOW				0x0040
#define VOH_CONTROL_WINDOW_NAME					0x0080
#define VOH_CONTROL_ICON_NAME					0x0100

/*--------------------------------------------------------------------------+
| Data types																|
+--------------------------------------------------------------------------*/

typedef struct {
	u_char	r, g, b;
} VOH_CLUT_ENTRY;

typedef struct {
    u_long		mask;
    u_char		number_of_bits_set;
    u_char		free_bits_at_bottom;
} VOH_RGB_MASK;

/* generic window id */
typedef void *VOH_WINDOW;

typedef struct {
	MSG_STRUCT *msg;
	const int (*send_event)(void *z, const void *event, size_t size);
	const int (*dith_class_init)(const void *z);
	const void (*dith_class_free)(const void *z);
	const void (*disp_refresh)(const void *z);
	const void (*notify_fd)(const void *z, int fd);
	const void *z;
	int handler_id;
} VOH_RESOURCES;

typedef struct {
	DITHER_TYPE dither_type;
	struct {
		double  horizontal;
		double  vertical;
	} scale;
	enum video_layout video_layout;
	double  pel_aspect_ratio;	/* 0 means free i.e. based on window geometry */
	struct {
		u_int horizontal;	/* mpeg Horizontal_size, from which Frame_width is derived */
		u_int vertical;		/* mpeg Vertical_size, from which Frame_height is derived */
		u_int frame_width;		/* reconstructed frame width in pels (always multiple of MB) */
		u_int frame_height;		/* reconstructed frame height in pels (always multiple of MB) */
	} size;
	struct {
		int		x;	/* used next time window is created/resized */
		int		y;	/* used next time window is created/resized */
	} window_position;
	VOH_WINDOW	existing_window;	/* if != 0, use existing window. -1 for root window */
	char window_name[PATH_MAX];
	char icon_name[PATH_MAX];
} VOH_CONTROL;

struct voh_data;

/*--------------------------------------------------------------------------+
| Externals																	|
+--------------------------------------------------------------------------*/


typedef struct {
	int	VOH_version;	/* handler must set this to SIH_VERSION */

	struct voh_data *(*VOH_init)(const VOH_RESOURCES *voh_resources, VOH_DITHERED_IMAGE *voh_dithered_image);
	void (*VOH_free)(struct voh_data *xinfo);

	int (*VOH_window_create)(struct voh_data *xinfo, VOH_CONTROL *voh_control, u_long flags);
	void (*VOH_window_destroy)(struct voh_data *xinfo);
	void (*VOH_window_raise)(struct voh_data *xinfo);

	void (*VOH_resolve_smart_dither)(struct voh_data *xinfo, VOH_WINDOW	existing_window, Bool *dither_ok, DITHER_TYPE *prefered_type_ptr);

	void (*VOH_read_and_handle_all_pending_events)(struct voh_data *xinfo);
#if 0 /* Tristan Savatier - Tue Mar 23 00:15:20 PST 1999 */
//	int (*VOH_get_fd)(struct voh_data *xinfo);
#endif /* 0 - Tristan Savatier - Tue Mar 23 00:15:20 PST 1999 */

	int (*VOH_dithered_image_alloc)(struct voh_data *xinfo);
	void (*VOH_dithered_image_free)(struct voh_data *xinfo);
	void (*VOH_dithered_image_show)(struct voh_data *xinfo);
	Bool (*VOH_dithered_image_lock)(struct voh_data *xinfo);
	void (*VOH_dithered_image_unlock)(struct voh_data *xinfo);

	void (*VOH_set_clut_private)(struct voh_data *xinfo, const VOH_CLUT_ENTRY *clut, u_int clut_size);
	u_int (*VOH_set_clut_shared)(struct voh_data *xinfo, const VOH_CLUT_ENTRY *clut, u_int clut_size);
	void (*VOH_free_clut_entries)(struct voh_data *xinfo);

	u_long *(*VOH_get_cm_pixel_values)(struct voh_data *xinfo);
	void (*VOH_get_rgb_masks)(struct voh_data *xinfo, VOH_RGB_MASK *masks);

	void (*VOH_set_video_mode)(struct voh_data *xinfo, VIDEO_MODE video_mode);

	int (*VOH_set_control)(struct voh_data *xinfo, VOH_CONTROL *voh_control, u_long flags);
	int (*VOH_cntl)(struct voh_data *xinfo, const void *ptr, u_int size);
} VOH_EXPOSED;


/*--------------------------------------------------------------------------+
| Externals																	|
+--------------------------------------------------------------------------*/
#ifdef __cplusplus
extern "C" {
#endif

extern int _init(void);
extern int _fini(void);
extern int	VOH_register(VOH_EXPOSED *exposed, int version);

#ifdef __cplusplus
}
#endif

#endif /* _voh_api_h */
