#ifndef lint
static char rcsid[] = "$Header: /net/hopkins/home/tristan/mpeg/dec/RCS/test_vci.c,v 1.4 1999/02/25 11:46:11 tristan Exp $";
static char copyright[] = "Copyright (c) 1997, MpegTV LLC";
#endif


/*
 * Disclaimer of Warranty
 *
 * These software programs are available to the user without any license fee or
 * royalty on an "as is" basis.  Copyright-holder disclaims
 * any and all warranties, whether express, implied, or statuary, including any
 * implied warranties or merchantability or of fitness for a particular
 * purpose.  In no event shall the copyright-holder be liable for any
 * incidental, punitive, or consequential damages of any kind whatsoever
 * arising from the use of these programs.
 *
 * This disclaimer of warranty extends to the user of these programs and user's
 * customers, employees, agents, transferees, successors, and assigns.
 *
 */


#include "mtvp/mtvp.h"

#include "../vci/sample_vci.h"

/***************************************************************************

This is a working example of application using a VCI plugin to capture
the reconstructed frames.

The source code of the VCI plugin used by the application is
in mtvp-sdk/vci/.

For details on the way the VCI plugin sample_vco.so and this program interact,
please read the comments in the sample_vci.c file, then read
the comments in this file.

***************************************************************************/

/*--------------------------------------------------------------------------+
| Configuration flags														|
+--------------------------------------------------------------------------*/
/* pathname of the vci plugin that we will use */
#define VCI_PLUGIN	"../vci/sample_vci.so"

#define SAVE_PPM_FILES	 /* save captured frame xxxx in file /tmp/xxxx.ppm */

/* #define CAPTURE_RATE 30  capture every N frame (default capture in real-time) */

/*--------------------------------------------------------------------------+
| Macros																	|
+--------------------------------------------------------------------------*/
/* for the msg.h package */
#define MSG_PTR (&Msg)

/*--------------------------------------------------------------------------+
| Data types																|
+--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------+
| Globals																	|
+--------------------------------------------------------------------------*/
/* for the msg.h package */
MSG_STRUCT Msg;

/*--------------------------------------------------------------------------+
| Statics																	|
+--------------------------------------------------------------------------*/

/* pid of this process */
static pid_t Pid;

/* to communicate with player */
static PMP_STRUCT	Player;

/*
 * Structure used by the VCI plugin to send us the Ids of the
 * shared memory segment and of the semaphore.
 */
static SHARED_IDS Shared_ids;

static Bool Done;	/* set to True when processing done */
static Bool Something_went_wrong;	/* set to True if something bad happened */


/* pointer to the shared memory segment once mapped */
static char *Shared_mem[SHMID_COUNT];

/* pointers to the frame YUV planes in the shared memory segment */
static char *Y_ptr[3], *U_ptr[3], *V_ptr[3];


/*--------------------------------------------------------------------------+
| init_shared_stuff															|
+--------------------------------------------------------------------------*/
static void
init_shared_stuff(shared_ids)
	SHARED_IDS *shared_ids;
{
	u_int	n;

	Shared_ids = *shared_ids;

		
#ifdef DEBUG
	printf("width = %u, height = %u\n", Shared_ids.width, Shared_ids.height);
#endif

	for (n = 0; n < SHMID_COUNT; n++) {
#ifdef DEBUG
		printf("Shared_ids.shmid[%d] = %d\n", n, Shared_ids.shmid[n]);
#endif
		assert(Shared_mem[n] == -1);
		
		/* attach shared memory segment */
		if ((Shared_mem[n] = shmat(Shared_ids.shmid[n], NULL, ACCESS_PERMS)) == (char *)-1) {
			FATAL_ERROR(("shmat"));
		}
#ifdef DEBUG
		printf("Shared_mem[%u] = %u\n", n, (u_int)Shared_mem[n]);
#endif
		/* make sure the segment will be destroyed once detached */
		if (shmctl(Shared_ids.shmid[n], IPC_RMID, NULL) == -1) {
			MESSAGE_ERROR(("shmctl"));
		}
		/* get pointers to the YUV planes */
		Y_ptr[n] = Shared_mem[n] + sizeof(YUV_IMAGE_AND_INDEX);
		U_ptr[n] = Y_ptr[n] + Shared_ids.width * Shared_ids.height;
		V_ptr[n] = U_ptr[n] + Shared_ids.width/2 * Shared_ids.height/2;
	}
}


/*--------------------------------------------------------------------------+
| process_frame																|
+--------------------------------------------------------------------------*/
static void
process_frame(n)
	u_int		n;
{
	const YUV_IMAGE *image;

	image = (const YUV_IMAGE *)Shared_mem[n];

	/*
	 * Process the frame that is in the shared memory segment.
	 */

	printf("width = %d, height = %d, index = %d\n", image->plane[(int)Y].width, image->plane[(int)Y].height, image->frame_infos.display_index);

	/* process frame only if frame size has not changed!! */
	if (Shared_ids.width == image->plane[(int)Y].width && Shared_ids.height == image->plane[(int)Y].height) {

#ifdef SAVE_PPM_FILES
		{
			char filename[PATH_MAX];
			
			sprintf(filename, "/tmp/%04d.ppm", image->frame_infos.display_index);
#ifdef DEBUG
			printf("saving frame in file %s\n", filename);
#endif
			if (store_ppm_tga (Y_ptr[n], V_ptr[n], U_ptr[n], filename, image->plane[(int)Y].height, image->plane[(int)Y].width, False) == -1) {
				MESSAGE_ERROR(("store_ppm_tga"));
			}
		}
#endif /* SAVE_PPM_FILES */

	} else {
		/*
		 * This can only happen when decoding concatenated
		 * streams with different frame sizes.  This example assumes
		 * that it is not the case, but we test that situation just in case...
		 */
		printf("something went wrong: frame size changed!\n");
		Something_went_wrong = True;
	}
#ifdef CAPTURE_RATE
	/* step N frames */
	g2a_send_int(&Player, COM_STEPS, 0, CAPTURE_RATE);
#endif

	/* send empty COM_VCI_CNTL to tell the player to grab the next frame */
	g2a_send_custom(&Player, COM_VCI_CNTL, 0, NULL, 0);
}


/*--------------------------------------------------------------------------+
| read_mtvp_message															|
+--------------------------------------------------------------------------*/
static int
read_mtvp_message()
{
	PMP_HEADER	msg;		/* to hold message header from Player */ 
	PMP_DATA_A2G	data;	/* to hold message data from Player */
		 
	/*
	 * Read message from player and process it.
	 */
	 
	/* read message from mtvp */ 
	if (a2g_read_msg(&Player, &msg, &data) == -1) {
		FATAL_ERROR(("a2g_read_msg: error reading msg"));
	}
	{
		char buf[PMP_PRINT_MSG_BUF_SIZE];
		/* MESSAGE() prints a message if (Msg_select & MSG_INFO) */
		MESSAGE(("read_mtvp_message: received msg: %s", pmp_print_msg(&msg,  (const char *)&data, buf)));
	}
	
	/* proces message */
	switch ((A2G_COM)msg.type) {
		case A2G_COM_API_VERSION:
			/* check API version */
			if (data.data_generic.data_int != MTVP_API_VERSION) {
				FATAL(("mtvp API version mismatch, expecting %d, got %d. The mtvp program found in your PATH is not compatible with this front-end!", MTVP_API_VERSION, data.data_generic.data_int)); 
			}
			break;
		case A2G_COM_VCI_FEEDBACK:
			/*
			 * Messages sent by the VCI plugin. Those messages use
			 * a private protocol agreed between the application and the plugin.
			 */
#ifdef DEBUG
			printf("received A2G_COM_VCI_FEEDBACK, size = %u\n", msg.size);
#endif
			/* zero-size message means that a frame is ready to be processed */
			if (msg.size == sizeof(int)) {
				/*
				 * The parameter is the index, and we will use it find the
				 * correct shared memory segment address in the aplication
				 * address space.
				 * Note that we should arrive there only after init_shared_stuff()
				 * has been called.
				 */
				process_frame(data.data_generic.data_int);				
				break;
			}

			/* attach the shared memory segment etc. */
			init_shared_stuff((SHARED_IDS *)&data);			
			break;
		case A2G_COM_CHALLENGE:
			printf("something went wrong: player could not find the license key.\n");
			printf("you need at least a temporary development key.\n");
			printf("see http://www.mpegtv.com/sdk.html for details\n");
			
			/* Something_went_wrong = True; */
			
			/*
			 * Note: It is generally better for an application to catch
			 * this license problem here, rather than 30 seconds later when mtvp
			 * exits with an error.
			 */ 
			break;
		case A2G_COM_QUIT:
			/*
			 * Sent by player right before it exits... Handling this message
			 * is futile since we alway get notified of the player's death by
			 * receiving a SIGCHLD signal.
			 */
			FATAL(("Player decided to quit (have you purchased the license?)"));
			break;
		case A2G_COM_STREAM_END:
			/* this means that the last frame has been processed */
#ifdef DEBUG
			printf("received A2G_COM_STREAM_END\n");
#endif
			Done = True;
			break;
		case A2G_COM_ALERT:
			{
				const PMP_DATA_ALERT *alert_data = (const PMP_DATA_ALERT *)&data;
				
				switch (alert_data->alert) {
					case ALERT_BAD_STREAM:
					case ALERT_MPEG2_VIDEO:
						printf("something went wrong: bad stream\n");
						Something_went_wrong = True;
						break;
					default:
						break;
				}
			}
			break;
		case A2G_COM_RETURN_FAILURE:
			/*
			 * This message is received each time a command that was sent with
			 * the PMP_FLAG_RETURN_STATUS flag failed.
			 * This is convenient to test the return status of commands like
			 * COM_VCI_LINK or COM_OPEN_FILE.  Note that if PMP_FLAG_RETURN_ORIGINAL
			 * was set too, a copy of the original message is sent back right
			 * after the A2G_COM_RETURN_FAILURE message.
			 */
			printf("something went wrong: command failed\n");
			Something_went_wrong = True;
			break;
		default:
			/* ignore other messages */
			break;
	}

	return 0;
}


/*--------------------------------------------------------------------------+
| clean_exit																|
+--------------------------------------------------------------------------*/
void
clean_exit(n)
	int		n;
{
	u_int	i;
	
	/* This is the routine called by the EXIT() macro */
	
	WARNING(("(pid=%u): exiting, status = %d", (u_int)Pid, n));
	
		
	for (i = 0; i < sizeofarray(Shared_mem); i++) {
		/* detach segment */
		if (Shared_mem[i] != (char *)-1) {
#ifdef DEBUG
			printf("detaching shared memory segment\n");
#endif
			if (shmdt(Shared_mem[i]) == -1) {
				MESSAGE_ERROR(("shmdt"));
			}
			Shared_mem[i] = (char *)-1;
		}
	}
	

#if 0 /* Tristan Savatier - Thu Jun 11 20:41:23 PDT 1998 */
//	/* a sure way to kill the Player... */
//	
//	/* make sure we ignore SIGCHLD signals */
//	(void)signal(SIGCHLD, SIG_IGN);
//	
//	/* send SIGTERM signal to Player */
//	if (Player.pid != 0) {
//		kill(Player.pid, SIGTERM);
//	}
#endif /* 0 - Tristan Savatier - Thu Jun 11 20:41:23 PDT 1998 */
	
	/* terminate (real exit this time) */
	exit(n);
}


#if 0 /* Tristan Savatier - Thu Jun 11 20:39:36 PDT 1998 */
///*--------------------------------------------------------------------------+
//| handler_sigchld															|
//+--------------------------------------------------------------------------*/
///*ARGSUSED*/
//static void
//handler_sigchld(sig)
//	int     sig;
//{
//	WARNING(("(pid=%u): received signal %d", (u_int)Pid, sig));
//
//	if (Player.pid != 0) {
//	
//		/* check if player actually died */
//		switch (g2a_child_died(&Player)) {
//		case 0:
//			WARNING(("(pid=%u): child process %u still alive, signal ignored", (u_int)Pid, (u_int)Player.pid));
//			break;
//		case 1:
//			WARNING(("(pid=%u): child process %u died, exiting...", (u_int)Pid, (u_int)Player.pid));
//			EXIT(EXIT_FAILURE);
//		case -1:
//			FATAL_ERROR(("(pid=%u): g2a_child_died(%u) error", (u_int)Pid, (u_int)Player.pid));
//		default:
//			FATAL(("What?!? g2a_child_died() returned bad value!"));
//		}
//	}
//
//#ifdef SIGNAL_RESETS_HANDLER
//	(void)signal(sig, handler_sigchld);
//#endif
//}
#endif /* 0 - Tristan Savatier - Thu Jun 11 20:39:36 PDT 1998 */

/*--------------------------------------------------------------------------+
| init																		|
+--------------------------------------------------------------------------*/
static void
init(argv0)
	const char *argv0;
{
	Shared_ids.shmid[0] = Shared_ids.shmid[1] = Shared_ids.shmid[2] = -1;

	/*
	 * Set Msg_select == -1 to print all the warnings and messages.
	 * See msg.h for the MESSAGE() and WARNING() macros definitions.
	 */
	 
#ifdef DEBUG
	Msg.select = -1;
#endif
	
#if 0 /* Tristan Savatier - Thu Jun 11 20:39:42 PDT 1998 */
//	(void)signal(SIGCHLD, handler_sigchld);
#endif /* 0 - Tristan Savatier - Thu Jun 11 20:39:42 PDT 1998 */

	Pid = getpid();

	MESSAGE(("started, pid = %u", (u_int)Pid));
	
	/*
	 * Start the player
	 */
	
	{
		static const char *mtvp_argv[] = {
			"mtvp",	/* name of the player */
			/* "-w", */	/* possible options */
			NULL,	/* must be NULL-terminated */
		};

		/*
		 * First parameter of g2a_init() is a ptr to a NULL-terminated array
		 * of strings.
		 * The first string is the name of the Player ("mtvp").
		 * The following strings are possible parameters.
		 * If the first string is a full pathname (starting with "/")
		 * g2a_init() will run this program as "the Player".
		 *
		 * If the first string is not a full pathname (starting with "/")
		 * g2a_init() will look for the Player in the following places:
		 * - in the directory where argv[0] is located (if argv[0] contains a '/')
		 * - in the current directory (".")
		 * - in the PATH
		 *
		 * Typically first string is "mtvp", if the player was installed
		 * under this name.
		 *
		 * Second parameter of g2a_init() is a string that should be
		 * the argv[0] from the calling program.
		 */
	
		if (g2a_init(&Player, mtvp_argv, argv0) == -1) {
			FATAL_ERROR(("Cannot fork player - check that mtvp is in your PATH!"));
		}
	}
}


/*--------------------------------------------------------------------------+
| main																		|
+--------------------------------------------------------------------------*/
/*ARGSUSED*/
int
main(argc, argv, envp)
	int argc;
	const char **argv, **envp;
{
#if 0 /* Tristan Savatier - Mon Oct 20 01:52:09 PDT 1997 */
//	key_t key;
//	const char *tmp_filename;
#endif /* 0 - Tristan Savatier - Mon Oct 20 01:52:09 PDT 1997 */


	/* expect one argument, assume that it is an MPEG file to open */
	if (argc != 2) {
		fprintf(stderr, "usage: test_vci file.mpg\n");
		clean_exit(EXIT_FAILURE);		
	}
	
#ifdef SAVE_PPM_FILES

	/* we need those initializations before calling store_ppm_tga */
	clip_init();
	yuv_to_rgb_init();
	
#endif /* SAVE_PPM_FILES */

	{
		u_int	i;
		
		for (i = 0; i < sizeofarray(Shared_mem); i++) {
			Shared_mem[i] = (char *)-1;
		}
	}

	/* start mtvp */
	init(argv[0]);
		
	/* link VCI plugin */
	g2a_send_string(&Player, COM_VCI_LINK, PMP_FLAG_RETURN_STATUS, VCI_PLUGIN);

#ifdef CAPTURE_RATE
	/* turn auto-play off */
	g2a_send_int(&Player, COM_AUTO_PLAY, 0, 0);
#else
	/* turn auto-play on */
	g2a_send_int(&Player, COM_AUTO_PLAY, 0, 1);
#endif

	/* open MPEG file */
	g2a_send_string(&Player, COM_OPEN_FILE, PMP_FLAG_RETURN_STATUS, argv[1]);

	/* send empty COM_VCI_CNTL */
	g2a_send_custom(&Player, COM_VCI_CNTL, 0, NULL, 0);
	
	MESSAGE(("entering main loop"));

	/* enter event loop */
	while (!Done && !Something_went_wrong) {
		/* read and process messages from player */
		read_mtvp_message();
	}
	
	/* test if anything went wrong */
	if (Something_went_wrong) {
		printf("something went wrong!\n");
		clean_exit(EXIT_FAILURE);
	}

	/* terminated successfully */
	clean_exit(EXIT_SUCCESS);

	/* so that lint does not complain! */
	return 0;
}


/* eof */
